#include "workflow_graph.hpp"

namespace xios
{

  std::unordered_map <StdString, vector <int > > *CWorkflowGraph::mapFieldToFilters_ptr = 0;
  #pragma omp threadprivate(CWorkflowGraph::mapFieldToFilters_ptr)

  std::unordered_map <int, StdString> *CWorkflowGraph::mapFilters_ptr = 0;
  #pragma omp threadprivate(CWorkflowGraph::mapFilters_ptr)

  std::unordered_map <int,vector<Time> > *CWorkflowGraph::mapFilterTimestamps_ptr = 0;
  #pragma omp threadprivate(CWorkflowGraph::mapFilterTimestamps_ptr)

  set<Time>  *CWorkflowGraph::timestamps_ptr = 0;
  #pragma omp threadprivate(CWorkflowGraph::timestamps_ptr)

  vector <StdString> CWorkflowGraph::filters;
  vector <StdString> CWorkflowGraph::fields;
  vector <pair<int, int> > CWorkflowGraph::fieldsToFilters;

  CWorkflowGraph::CWorkflowGraph()
  { }

  /*!
   * Reorganize information collected by XIOS for visualization.
   */
  void CWorkflowGraph::buildStaticWorkflow()
  TRY
  {
    // Create a list of filters (graph nodes)
    if(mapFilters_ptr==0) mapFilters_ptr = new std::unordered_map <int, StdString>;
    if(mapFieldToFilters_ptr==0) mapFieldToFilters_ptr = new std::unordered_map <StdString, vector <int > >;
    size_t filterIdx = 0;
    filters.resize(mapFilters_ptr->size());
    //printf("mapFilters_ptr size = %lu\n", mapFilters_ptr->size());
    for (auto it=mapFilters_ptr->begin(); it != mapFilters_ptr->end(); it++)
    {
      filters[it->first]=it->second;
      //std::cout<<"mapFilters_ptr["<<it->first<<"]="<<it->second<<std::endl;
    }

    // Create a list of fields and their filter connectivity (graph edges and edge-to-node connectivity)
    for (auto it=mapFieldToFilters_ptr->begin(); it != mapFieldToFilters_ptr->end(); it++)
    {
      for (size_t i = 0; i < (it->second.size() - 1); i+=2)
      {
        fieldsToFilters.push_back(make_pair(it->second[i],it->second[i+1]));
        fields.push_back(it->first);
      }
    }
  }
  CATCH

}
